/*
 * Limits on FFS filenames and pathnames
 *
 * The deepest pathname allowed is one of the form /1/2/3/4/5/6, where the
 * last component may be a file, a directory or a symlink; if this last
 * component is a directory, it has to be empty, because any child of
 * that directory would violate the depth limit.
 *
 * The proper FFS pathname form begins with a slash (all pathnames must
 * be absolute, no Unix processes in the fw means no current directories),
 * has exactly one slash in each separating place (no double slashes),
 * and no trailing slash except in the special case of the root directory,
 * whose full pathname is "/".
 *
 * Each component name is [1,20] characters long; combining this limit
 * with the maximum depth of 6 puts the maximum length of a properly-formed
 * full pathname at 126 characters.
 */

#define	MAX_FN_COMPONENT	20
#define	MAX_NAME_DEPTH		6
#define	MAX_FULL_PATHNAME	((MAX_FN_COMPONENT+1) * MAX_NAME_DEPTH)
